<?php 
namespace Concrete\Package\Checklist\Block\Checklist;

use \Concrete\Core\Block\BlockController;
use \Database;
use \Core;
use Symfony\Component\HttpFoundation\JsonResponse;

defined('C5_EXECUTE') or die("Access Denied.");

class Controller extends BlockController
{

    protected $btTable = 'btChecklist';
    protected $btInterfaceWidth = "500";
    protected $btInterfaceHeight = "400";
    protected $btCacheBlockRecord = true;
    protected $btCacheBlockOutput = true;
    protected $btCacheBlockOutputOnPost = true;
    protected $btCacheBlockOutputLifetime = 0; //until manually updated or cleared
    protected $btDefaultSet = 'form';

    public function getBlockTypeDescription()
    {
        return t("Add a collaborative checklist to a page.");
    }

    public function getBlockTypeName()
    {
        return t("Checklist");
    }

    public function registerViewAssets()
    {
        $this->requireAsset('javascript', 'jquery');
    }

    public function add()
    {
        $this->set('checkAccess', 'E');
        $this->set('items', json_encode(array()));
    }

    public function edit()
    {
        $this->set('items', json_encode($this->getChecklistItems()));
    }

    public function action_check($bID)
    {
        if ($bID == $this->bID) {
            $id = intval($this->post('id'));
            if ($id) {
                $items = $this->getChecklistItems();
                foreach($items as $item) {
                    if ($item->id == $id) {
                        break;
                    }
                }
            }

            if (isset($item) && $this->canCheckItem($item)) {
                $db = \Loader::db();
                $checked = intval($this->post('checked'));
                $db->update('btChecklistItems', array('isChecked' => $checked),
                    array('bID' => $this->bID, 'btChecklistItemID' => $item->id));

                $o = new \stdClass;
                $o->bID = $this->bID;
                $o->checked = $checked;
                $o->id = $item->id;
                $response = new JsonResponse($o);
                return $response;
            }


        }

        $this->view();
    }

    protected function getChecklistItems($encode = false)
    {
        $db = \Database::get();
        $r = $db->GetAll('select btChecklistItemID, isChecked, item from btChecklistItems where bID = ? order by displayOrder asc',
            array($this->bID)
        );
        $items = array();
        foreach($r as $item) {
            $o = new \stdClass;
            $o->id = $item['btChecklistItemID'];
            $o->isChecked = $item['isChecked'];
            $o->item = $encode ? h($item['item']) : $item['item'];
            $items[] = $o;
        }
        return $items;
    }

    public function canCheckItem($item)
    {
        $c = \Page::getCurrentPage();
        $cp = new \Permissions($c);
        switch($this->checkAccess) {
            case 'E':
                $b = $this->getBlockObject();
                $bp = new \Permissions($b);
                return $bp->canEditBlock();
            case 'R':
                $u = new \User();
                return $cp->canViewPage() && $u->isRegistered();
            case 'A':
                return $cp->canViewPage();
        }
    }

    public function duplicate($newBlockID)
    {
        parent::duplicate($newBlockID);
        $db = \Database::get();
        $i = 0;
        foreach($this->getChecklistItems() as $item) {
            $db->insert('btChecklistItems', array(
                'btChecklistItemID' => $item->id,
                'bID' => $newBlockID,
                'item' => $item->item,
                'displayOrder' => $i));
            $i++;
        }
    }

    public function validate()
    {
        $e = Core::make('helper/validation/error');
        $item = $this->post('item');
        if (count($item) == 0) {
            $e->add(t('Your check list must contain at least one item.'));
        }
        return $e;
    }

    public function save($args)
    {
        parent::save($args);
        $db = Database::get();
        $items = $args['item'];
        $displayOrder = 0;
        foreach($items as $index => $item) {
            if (isset($args['checklistItemID'][$index]) && $args['checklistItemID'][$index]) {
                // this is an update of an existing one.
                $db->Execute('update btChecklistItems set item = ?, displayOrder = ? where bID = ? and btChecklistItemID = ?',
                    array(
                        $item, $displayOrder, $this->bID, $args['checklistItemID'][$index]
                    )
                );
            } else {
                $checklistItemID = $db->GetOne('select max(btChecklistItemID) from btChecklistItems where bID = ?',
                    array($this->bID)
                );
                if (!$checklistItemID) {
                    $checklistItemID = 0;
                }
                $checklistItemID++;

                $db->insert('btChecklistItems',
                    array('item' => $item, 'displayOrder' => $displayOrder,
                        'bID' => $this->bID, 'btChecklistItemID' => $checklistItemID)
                );
            }
            $displayOrder++;
        }

    }

    public function delete()
    {
        parent::delete();
        $db = Database::get();
        $db->delete('btChecklistItems', array('bID' => $this->bID));
    }

    public function view()
    {
        $this->set('items', $this->getChecklistItems(true));
    }

}